@@ -4,30 +4,52 @@
     SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
-#include "usermetadatawritertest.h"
 #include "indexerextractortestsconfig.h"
 #include "usermetadata.h"
 
 #include <QTest>
 #include <QFile>
 
-#define TEST_FILENAME "writertest-usermetadata.txt"
-#define TEST_SYMLINK "dangling_symlink-metadata"
+namespace {
+const auto TEST_FILENAME = QStringLiteral("writertest-usermetadata.txt");
+const auto TEST_SYMLINK = QStringLiteral("dangling_symlink-metadata");
+
+QString testOutputPath(const QString& fileName)
+{
+    return QStringLiteral(INDEXER_TESTS_OUTPUT_PATH "/UserMetaDataWriterTest/%1").arg(fileName);
+}
+}
+
+class UserMetaDataWriterTest : public QObject
+{
+    Q_OBJECT
+
+private Q_SLOTS:
+    void initTestCase();
+    void test();
+    void testMissingPermision();
+    void testMetadataSize();
+    void testMetadataNameTooLong();
+    void testDanglingSymlink();
+    void testRemoveMetadata();
+    void testMetadataFolder();
+    void cleanupTestCase();
+
+private:
+    QFile m_writerTestFile;
+};
 
 using namespace KFileMetaData;
 
-QString UserMetaDataWriterTest::testFilePath(const QString& fileName) const
-{
-    return QLatin1String(INDEXER_TESTS_SAMPLE_FILES_PATH) + QLatin1Char('/') + fileName;
-}
-
 void UserMetaDataWriterTest::initTestCase()
 {
-    m_writerTestFile.setFileName(testFilePath(TEST_FILENAME));
+    QVERIFY(QDir().mkpath(testOutputPath({})));
+
+    m_writerTestFile.setFileName(testOutputPath(TEST_FILENAME));
     auto opened = m_writerTestFile.open(QIODevice::WriteOnly | QIODevice::NewOnly);
     QVERIFY(opened);
 
-    QFile::link(testFilePath("invalid_target"), testFilePath(TEST_SYMLINK));
+    QFile::link(testOutputPath(QStringLiteral("invalid_target")), testOutputPath(TEST_SYMLINK));
 }
 
 void UserMetaDataWriterTest::testMissingPermision()
@@ -36,10 +58,10 @@ void UserMetaDataWriterTest::testMissingPermision()
     QSKIP("Only unix permissions can restrict metadata writing");
 #endif
     m_writerTestFile.setPermissions(QFileDevice::ReadOwner);
-    KFileMetaData::UserMetaData md(testFilePath(TEST_FILENAME));
+    KFileMetaData::UserMetaData md(testOutputPath(TEST_FILENAME));
     QVERIFY(md.isSupported());
 
-    auto result = md.setAttribute("test", "my-value");
+    auto result = md.setAttribute(QStringLiteral("test"), QStringLiteral("my-value"));
     QCOMPARE(result, KFileMetaData::UserMetaData::MissingPermission);
 
     QVERIFY(m_writerTestFile.setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner));
@@ -47,7 +69,7 @@ void UserMetaDataWriterTest::testMissingPermision()
 
 void UserMetaDataWriterTest::testMetadataSize()
 {
-    KFileMetaData::UserMetaData md(testFilePath(TEST_FILENAME));
+    KFileMetaData::UserMetaData md(testOutputPath(TEST_FILENAME));
     QVERIFY(md.isSupported());
 
     // In the current ext2, ext3, and ext4 filesystem implementations,
@@ -55,32 +77,38 @@ void UserMetaDataWriterTest::testMetadataSize()
     // extended attributes must fit in a single filesystem block (1/2/4 kB)
 
     // all implementations should support at least 512 B
-    const auto smallSize = 512; // 512 B
-    auto smallValue = QString(smallSize, 'a');
-    auto result = md.setAttribute("test", smallValue);
+    constexpr auto smallSize = 512; // 512 B
+    auto smallValue = QString(smallSize, QLatin1Char('a'));
+    auto result = md.setAttribute(QStringLiteral("test"), smallValue);
     QCOMPARE(result, KFileMetaData::UserMetaData::NoError);
-    QCOMPARE(md.attribute("test"), smallValue);
+    QCOMPARE(md.attribute(QStringLiteral("test")), smallValue);
 
-    // a big value, equal to the maximum value of an extended attribute according to Linux VFS
-    // applies to XFS, btrfs...
-    auto maxSize = 64 * 1024;
-    const auto bigValue = QString(maxSize, 'a'); // 64 kB
-    result = md.setAttribute("test", bigValue);
+    // a big value, maxing-out the VFS-imposed limit on Linux
+    constexpr auto maxSize = 64 * 1024;
+    const auto bigValue = QString(maxSize, QLatin1Char('a')); // 64 kB
+    result = md.setAttribute(QStringLiteral("test"), bigValue);
 #if defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD) || defined(Q_OS_WIN)
     // BSD VFS has no such limit to 64 kB
     QCOMPARE(result, KFileMetaData::UserMetaData::NoError);
-    QCOMPARE(md.attribute("test"), bigValue);
+    QCOMPARE(md.attribute(QStringLiteral("test")), bigValue);
 #else
-    QCOMPARE(result, KFileMetaData::UserMetaData::NoSpace);
+    if (result == KFileMetaData::UserMetaData::NoSpace) {
+        // FS specific limit: nodesize (16 kB) for BTRFS, blocksize on Ext*, ...
+        QCOMPARE(result, KFileMetaData::UserMetaData::NoSpace);
+    } else {
+        // No FS specific limit, full VFS size supported, e.g. XFS
+        QCOMPARE(result, KFileMetaData::UserMetaData::NoError);
+        QCOMPARE(md.attribute(QStringLiteral("test")), bigValue);
+    }
 #endif
 
-    // In Linux, The VFS-imposed limits on attribute names and
+    // On Linux, the VFS-imposed limits on attribute names and
     // values are 255 bytes and 64 kB, respectively.
-    auto excessiveValue = QString(maxSize + 1, 'a');
-    result = md.setAttribute("test", excessiveValue);
+    auto excessiveValue = QString(maxSize + 1, QLatin1Char('a'));
+    result = md.setAttribute(QStringLiteral("test"), excessiveValue);
 #if defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD) || defined(Q_OS_WIN)
     QCOMPARE(result, KFileMetaData::UserMetaData::NoError);
-    QCOMPARE(md.attribute("test"), excessiveValue);
+    QCOMPARE(md.attribute(QStringLiteral("test")), excessiveValue);
 #else
     // In Linux, we exceed the max value of an extended attribute, the error is different
     QCOMPARE(result, KFileMetaData::UserMetaData::ValueTooBig);
@@ -89,19 +117,19 @@ void UserMetaDataWriterTest::testMetadataSize()
 
 void UserMetaDataWriterTest::testMetadataNameTooLong()
 {
-    KFileMetaData::UserMetaData md(testFilePath(TEST_FILENAME));
+    KFileMetaData::UserMetaData md(testOutputPath(TEST_FILENAME));
     QVERIFY(md.isSupported());
 
     // BSD and Linux have a limit of the attribute name of 255 bytes
     // Windows has by default a limit on filename that applies to filesystem metadata
-    auto longName = QString(256, 'a');
-    int result = md.setAttribute(longName, "smallValue");
+    auto longName = QString(256, QLatin1Char('a'));
+    int result = md.setAttribute(longName, QStringLiteral("smallValue"));
     QCOMPARE(result, KFileMetaData::UserMetaData::NameToolong);
 }
 
 void UserMetaDataWriterTest::test()
 {
-    auto testFile = testFilePath(TEST_FILENAME);
+    auto testFile = testOutputPath(TEST_FILENAME);
     KFileMetaData::UserMetaData md(testFile);
     QVERIFY(md.isSupported());
 
@@ -136,8 +164,8 @@ void UserMetaDataWriterTest::test()
     QVERIFY(!md.hasAttribute(QStringLiteral("xdg.comment")));
 
     // Origin url
-    md.setOriginUrl(QUrl("http://this.is.a.test.website.local"));
-    QCOMPARE(md.originUrl(), QUrl("http://this.is.a.test.website.local"));
+    md.setOriginUrl(QUrl(QStringLiteral("http://this.is.a.test.website.local")));
+    QCOMPARE(md.originUrl(), QUrl(QStringLiteral("http://this.is.a.test.website.local")));
     md.setOriginUrl(QUrl());
     QVERIFY(!md.hasAttribute(QStringLiteral("xdg.origin.url")));
 
@@ -183,13 +211,13 @@ void UserMetaDataWriterTest::test()
 
 void UserMetaDataWriterTest::testDanglingSymlink()
 {
-    KFileMetaData::UserMetaData md(testFilePath(TEST_SYMLINK));
+    KFileMetaData::UserMetaData md(testOutputPath(TEST_SYMLINK));
     QVERIFY(md.queryAttributes(UserMetaData::Attribute::All) == UserMetaData::Attribute::None);
 }
 
 void UserMetaDataWriterTest::testRemoveMetadata()
 {
-    auto testFile = testFilePath(TEST_FILENAME);
+    auto testFile = testOutputPath(TEST_FILENAME);
     KFileMetaData::UserMetaData md(testFile);
     QVERIFY(md.isSupported());
 
@@ -203,7 +231,7 @@ void UserMetaDataWriterTest::testRemoveMetadata()
 
 void UserMetaDataWriterTest::testMetadataFolder()
 {
-    const auto dirPath = testFilePath(QStringLiteral("metadata-dir"));
+    const auto dirPath = testOutputPath(QStringLiteral("metadata-dir"));
     QVERIFY(QDir().mkdir(dirPath));
 
     KFileMetaData::UserMetaData md(dirPath);
@@ -214,14 +242,22 @@ void UserMetaDataWriterTest::testMetadataFolder()
 
     QCOMPARE(md.setAttribute(QStringLiteral("tag"), QString{}), KFileMetaData::UserMetaData::NoError);
     QVERIFY(!md.hasAttribute(QStringLiteral("tag")));
+
+    // Removing a non-existing tag is not an error
+    QCOMPARE(md.setAttribute(QStringLiteral("tag"), QString{}), KFileMetaData::UserMetaData::NoError);
+    QVERIFY(!md.hasAttribute(QStringLiteral("tag")));
+
+    QVERIFY(QDir().rmdir(dirPath));
 }
 
 void UserMetaDataWriterTest::cleanupTestCase()
 {
     m_writerTestFile.remove();
-    QFile::remove(testFilePath(TEST_SYMLINK));
+    QFile::remove(testOutputPath(TEST_SYMLINK));
+
+    QVERIFY(QDir(testOutputPath({})).removeRecursively());
 }
 
 QTEST_GUILESS_MAIN(UserMetaDataWriterTest)
 
-#include "moc_usermetadatawritertest.cpp"
+#include "usermetadatawritertest.moc"
